function D = Decorrele(M)
  % Decorrele(M)
  %
  % Calcule une famille de n signaux décorrélés, centrés et de variance 1
  % à partir d'une famille de signaux (vecteurs de taille m)
  % On suppose que n>m, sinon les signaux ne sont certainement pas linéairement indépendants

  D = M;            % initialisation avec la même matrice
  % qu’on modifie petit à petit

  n = size(M,2);    % nombre de vecteurs colonnes

  for i = 1:n
    % On suppose que les vecteurs 1...(i-1) dans D sont décorrélés entre eux,
    % centrés et de variance 1
    % Et que les vecteurs i...n dans D sont décorrélés des vecteurs 1...(i-1) dans D
    aux = D(:,i);
    aux = (aux-mean(aux))/sqrt(var(aux));  % le vecteur normalisé centré

    D(:,i) = aux;  % on place ce vecteur dans D

    % On décorrèle les vecteurs i+1...n avec le vecteur i
    for j = (i+1):n

      tmp = D(:,j);

      covariance = cov(tmp, aux);

      % pour la compatibilité Matlab / Octave
      % après ce test, covariance contient Cov(tmp, aux)
      if numel(covariance) > 1
	covariance = covariance(2);
      end

      D(:,j) = tmp - mean(tmp) - covariance * aux;
      % D(:,j) est bien centré, et décorrélé avec aux puisque
      % cov (D(:,j),aux) = cov(tmp,aux) - 0 - cov(tmp,aux) * cov(aux,aux) = 0
    end
  end
end
